@@ -228,6 +228,7 @@ int main(int argc, char *argv[])
 	__connman_detect_init();
 	__connman_session_init();
 	__connman_timeserver_init();
+	__connman_connection_init();
 
 	__connman_plugin_init(option_plugin, option_noplugin);
 
@@ -249,6 +250,7 @@ int main(int argc, char *argv[])
 
 	__connman_plugin_cleanup();
 
+	__connman_connection_cleanup();
 	__connman_timeserver_cleanup();
 	__connman_session_cleanup();
 	__connman_detect_cleanup();
