@@ -501,7 +501,8 @@ int connman_network_set_associating(struct connman_network *network,
 
 		service = __connman_service_lookup_from_network(network);
 		__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_ASSOCIATION);
+					CONNMAN_SERVICE_STATE_ASSOCIATION,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
 	}
 
 	return 0;
@@ -519,7 +520,8 @@ static void set_associate_error(struct connman_network *network)
 	service = __connman_service_lookup_from_network(network);
 
 	__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_FAILURE);
+					CONNMAN_SERVICE_STATE_FAILURE,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
 }
 
 static void set_configure_error(struct connman_network *network)
@@ -531,7 +533,18 @@ static void set_configure_error(struct connman_network *network)
 	service = __connman_service_lookup_from_network(network);
 
 	__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_FAILURE);
+					CONNMAN_SERVICE_STATE_FAILURE,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
+}
+
+static void set_invalid_key_error(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
+
+	__connman_service_indicate_error(service,
+					CONNMAN_SERVICE_ERROR_INVALID_KEY);
 }
 
 void connman_network_set_ipv4_method(struct connman_network *network,
@@ -588,6 +601,9 @@ void connman_network_set_error(struct connman_network *network,
 	case CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL:
 		set_configure_error(network);
 		break;
+	case CONNMAN_NETWORK_ERROR_INVALID_KEY:
+		set_invalid_key_error(network);
+		break;
 	}
 }
 
@@ -621,7 +637,8 @@ static void set_configuration(struct connman_network *network)
 
 	service = __connman_service_lookup_from_network(network);
 	__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_CONFIGURATION);
+					CONNMAN_SERVICE_STATE_CONFIGURATION,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
 }
 
 static int set_connected_fixed(struct connman_network *network)
@@ -669,6 +686,7 @@ static void set_connected_manual(struct connman_network *network)
 	struct connman_service *service;
 	struct connman_ipconfig *ipconfig;
 	const char *nameserver = NULL;
+	const char *gateway;
 	int err;
 
 	DBG("network %p", network);
@@ -689,15 +707,19 @@ static void set_connected_manual(struct connman_network *network)
 	connman_element_get_value(&network->element,
 			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
 	if (nameserver != NULL)
-		__connman_service_append_nameserver(service, nameserver);
+		__connman_service_nameserver_append(service, nameserver);
 
-	__connman_ipconfig_set_gateway(ipconfig, &network->element);
+	connman_element_get_value(&network->element,
+				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
+	if (gateway != NULL)
+		__connman_ipconfig_set_gateway(ipconfig, gateway);
 
 	network->connecting = FALSE;
 
 	connman_network_set_associating(network, FALSE);
 
-	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY);
+	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
 }
 
 static int set_connected_dhcp(struct connman_network *network)
@@ -750,6 +772,36 @@ static int manual_ipv6_set(struct connman_network *network,
 	return 0;
 }
 
+static void autoconf_ipv6_set(struct connman_network *network)
+{
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+	const char *nameserver = NULL;
+
+	DBG("network %p", network);
+
+	service = __connman_service_lookup_from_network(network);
+
+	ipconfig = __connman_service_get_ip6config(service);
+
+	__connman_device_increase_connections(network->device);
+
+	__connman_device_set_network(network->device, network);
+
+	connman_device_set_disconnected(network->device, FALSE);
+
+	connman_element_get_value(&network->element,
+			CONNMAN_PROPERTY_ID_IPV6_NAMESERVER, &nameserver);
+	if (nameserver != NULL)
+		__connman_service_nameserver_append(service, nameserver);
+
+	network->connecting = FALSE;
+
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_READY,
+					CONNMAN_IPCONFIG_TYPE_IPV6);
+}
+
 static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
@@ -777,7 +829,9 @@ static gboolean set_connected(gpointer user_data)
 		switch (ipv6_method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+			break;
 		case CONNMAN_IPCONFIG_METHOD_AUTO:
+			autoconf_ipv6_set(network);
 			break;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 		case CONNMAN_IPCONFIG_METHOD_MANUAL:
@@ -826,7 +880,19 @@ static gboolean set_connected(gpointer user_data)
 		service = __connman_service_lookup_from_network(network);
 
 		__connman_service_indicate_state(service,
-						CONNMAN_SERVICE_STATE_IDLE);
+					CONNMAN_SERVICE_STATE_IDLE,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
+
+		/* TODO: eventually the IPv6 disconnect state should be handled
+		 * in connection.c
+		 */
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_DISCONNECT,
+					CONNMAN_IPCONFIG_TYPE_IPV6);
+
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_IDLE,
+					CONNMAN_IPCONFIG_TYPE_IPV6);
 	}
 
 	network->connecting = FALSE;
@@ -998,8 +1064,6 @@ static int dhcp_start(struct connman_network *network)
 
 static int dhcp_stop(struct connman_network *network)
 {
-	connman_element_unregister_children_type(&network->element,
-					CONNMAN_ELEMENT_TYPE_CONNECTION);
 	connman_element_unregister_children_type(&network->element,
 						CONNMAN_ELEMENT_TYPE_IPV4);
 	connman_element_unregister_children_type(&network->element,
@@ -1012,6 +1076,7 @@ static int manual_ipv4_set(struct connman_network *network,
 				struct connman_ipconfig *ipconfig)
 {
 	struct connman_service *service;
+	const char *gateway;
 	int err;
 
 	service = __connman_service_lookup_from_network(network);
@@ -1025,9 +1090,13 @@ static int manual_ipv4_set(struct connman_network *network,
 		return err;
 	}
 
-	__connman_ipconfig_set_gateway(ipconfig, &network->element);
+	connman_element_get_value(&network->element,
+				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
+	if (gateway != NULL)
+		__connman_ipconfig_set_gateway(ipconfig, gateway);
 
-	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY);
+	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
 
 	return 0;
 }
@@ -1037,12 +1106,14 @@ int __connman_network_clear_ipconfig(struct connman_network *network,
 {
 	struct connman_service *service;
 	enum connman_ipconfig_method method;
+	enum connman_ipconfig_type type;
 
 	service = __connman_service_lookup_from_network(network);
 	if (service == NULL)
 		return -EINVAL;
 
 	method = __connman_ipconfig_get_method(ipconfig);
+	type = __connman_ipconfig_get_config_type(ipconfig);
 
 	switch (method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
@@ -1051,8 +1122,6 @@ int __connman_network_clear_ipconfig(struct connman_network *network,
 	case CONNMAN_IPCONFIG_METHOD_AUTO:
 		return -EINVAL;
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
-		connman_element_unregister_children_type(&network->element,
-					CONNMAN_ELEMENT_TYPE_CONNECTION);
 		__connman_ipconfig_clear_address(ipconfig);
 		break;
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
@@ -1060,8 +1129,14 @@ int __connman_network_clear_ipconfig(struct connman_network *network,
 		break;
 	}
 
-	__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_CONFIGURATION);
+	if (type == CONNMAN_IPCONFIG_TYPE_IPV6)
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION,
+					CONNMAN_IPCONFIG_TYPE_IPV6);
+	else if (type == CONNMAN_IPCONFIG_TYPE_IPV4)
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION,
+					CONNMAN_IPCONFIG_TYPE_IPV4);
 
 	return 0;
 }
@@ -1079,7 +1154,9 @@ int __connman_network_set_ipconfig(struct connman_network *network,
 		switch (method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+			break;
 		case CONNMAN_IPCONFIG_METHOD_AUTO:
+			autoconf_ipv6_set(network);
 			break;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 		case CONNMAN_IPCONFIG_METHOD_MANUAL:
